import requests
from django import forms
from django.utils.translation import gettext_lazy as _

from capeditor.caputils import cap_xml_to_alert_data
from capeditor.errors import CAPImportError


class CAPLoadForm(forms.Form):
    LOAD_FROM_CHOICES = (
        ('text', _('Copy Paste XML')),
        ('url', _('URL')),
        ('file', _('File')),
    )

    load_from = forms.ChoiceField(choices=LOAD_FROM_CHOICES, initial="text", widget=forms.RadioSelect,
                                  label=_('Load from'), )
    text = forms.CharField(required=False, widget=forms.Textarea, label=_('Paste your CAP XML here'))
    url = forms.URLField(required=False, label=_('CAP Alert XML URL'))
    file = forms.FileField(required=False, label=_('CAP File'))

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['file'].widget.attrs.update({'accept': '.xml'})

    def clean(self):
        cleaned_data = super().clean()
        load_from = cleaned_data.get('load_from')
        text = cleaned_data.get('text')
        url = cleaned_data.get('url')
        file = cleaned_data.get('file')

        # field validation
        if load_from == 'text' and not text:
            self.add_error('text', _('This field is required.'))
        if load_from == 'url' and not url:
            self.add_error('url', _('This field is required.'))
        if load_from == 'file' and not file:
            self.add_error('file', _('This field is required.'))

        try:
            if load_from == 'text':
                content = cleaned_data.get('text')
                alert_source = {
                    "type": "Copied XML",
                }
            elif load_from == 'file':
                file = cleaned_data.get('file')
                content = file.read().decode('utf-8')
                alert_source = {
                    "name": file.name,
                    "type": "File",
                }
            else:
                url = cleaned_data.get('url')
                res = requests.get(url)
                res.raise_for_status()
                content = res.text
                alert_source = {
                    "name": url,
                    "type": "URL",
                }

            alert_data = cap_xml_to_alert_data(content)

            alert_data['alert_source'] = alert_source

            cleaned_data['alert_data'] = alert_data

        except CAPImportError as e:
            self.add_error(None, e.message)
            return cleaned_data

        except Exception as e:
            self.add_error(None, _("An error occurred while trying to load the CAP XML. "
                                   "Please check the data and try again."))
            return cleaned_data

        return cleaned_data


class CAPImportForm(forms.Form):
    alert_data = forms.JSONField(widget=forms.HiddenInput)



# Generated by Django 4.2.11 on 2024-04-29 07:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('capeditor', '0004_predefinedalertarea'),
    ]

    operations = [
        migrations.AlterField(
            model_name='capsetting',
            name='sender',
            field=models.CharField(blank=True, help_text='Email of the sending institution', max_length=255, null=True, verbose_name='CAP Sender Email'),
        ),
    ]



# Generated by Django 4.1.10 on 2024-02-07 15:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailimages', '0025_alter_image_file_alter_rendition_file'),
        ('capeditor', '0002_alter_capsetting_options_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='capsetting',
            name='logo',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.image', verbose_name='Logo of the sending institution'),
        ),
    ]



from django.shortcuts import render, redirect
from wagtail import hooks

from capeditor.forms.capimporter import CAPLoadForm, CAPImportForm


def load_cap_alert(request):
    load_template_name = "capeditor/load_cap_alert.html"
    preview_template_name = "capeditor/preview_cap_alert.html"

    context = {}

    if request.method == "POST":
        form = CAPLoadForm(request.POST, request.FILES)
        if form.is_valid():
            alert_data = form.cleaned_data["alert_data"]
            form = CAPImportForm(initial={"alert_data": alert_data})
            context.update({
                "alert_data": alert_data,
                "form": form,
            })

            return render(request, preview_template_name, context)

        else:
            context.update({"form": form})
            return render(request, template_name=load_template_name, context=context)

    form = CAPLoadForm()
    context.update({"form": form})

    return render(request, load_template_name, context)


def import_cap_alert(request):
    if request.method == "POST":
        form = CAPImportForm(request.POST)
        if form.is_valid():
            alert_data = form.cleaned_data["alert_data"]

            # run hook to import alert
            for fn in hooks.get_hooks("before_import_cap_alert"):
                result = fn(request, alert_data)
                if hasattr(result, "status_code"):
                    return result

            return redirect("load_cap_alert")
        else:
            return redirect("load_cap_alert")

    return redirect("load_cap_alert")



import uuid

from django.utils import timezone
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from shapely.geometry import shape
from wagtail import blocks
from wagtail.admin.forms import WagtailAdminPageForm
from wagtail.admin.panels import MultiFieldPanel
from wagtail.models import Page

from capeditor.blocks import (
    AlertInfo,
    SENDER_NAME_HELP_TEXT,
    CONTACT_HELP_TEXT,
    AUDIENCE_HELP_TEXT,
    AlertAddress,
    AlertReference,
    AlertIncident
)
from capeditor.constants import SEVERITY_MAPPING, URGENCY_MAPPING, CERTAINTY_MAPPING
from .cap_settings import *


class CapAlertPageForm(WagtailAdminPageForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        cap_setting = get_cap_setting()

        # the following code gets pres-saved common options from settings and
        # makes presents them as selectable dropdowns instead of the composer to type each time
        # they are creating a new alert
        if cap_setting:
            default_sender_name = cap_setting.sender_name
            contacts = cap_setting.contacts
            audience_types = cap_setting.audience_types

            if default_sender_name:
                info_field = self.fields.get("info")
                for block_type, block in info_field.block.child_blocks.items():
                    if block_type == "alert_info":
                        field_name = "senderName"
                        sender_block = info_field.block.child_blocks[block_type].child_blocks[field_name]

                        label = sender_block.label or field_name
                        name = sender_block.name

                        info_field.block.child_blocks[block_type].child_blocks[field_name] = blocks.CharBlock(
                            default=default_sender_name, required=False, help_text=SENDER_NAME_HELP_TEXT)
                        info_field.block.child_blocks[block_type].child_blocks[field_name].name = name
                        info_field.block.child_blocks[block_type].child_blocks[field_name].label = label

            if contacts:
                contact_choices = []
                for block in contacts:
                    contact = block.value.get("contact")
                    contact_choices.append((contact, contact))

                info_field = self.fields.get("info")
                for block_type, block in info_field.block.child_blocks.items():
                    if block_type == "alert_info":
                        field_name = "contact"
                        contact_block = info_field.block.child_blocks[block_type].child_blocks[field_name]

                        label = contact_block.label or field_name
                        name = contact_block.name

                        info_field.block.child_blocks[block_type].child_blocks[field_name] = blocks.ChoiceBlock(
                            choices=contact_choices, required=False, help_text=CONTACT_HELP_TEXT)
                        info_field.block.child_blocks[block_type].child_blocks[field_name].name = name
                        info_field.block.child_blocks[block_type].child_blocks[field_name].label = label

            if audience_types:
                audience_type_choices = []

                for block in audience_types:
                    audience = block.value.get("audience")
                    audience_type_choices.append((audience, audience))

                info_field = self.fields.get("info")
                for block_type, block in info_field.block.child_blocks.items():
                    if block_type == "alert_info":
                        field_name = "audience"
                        audience_block = info_field.block.child_blocks[block_type].child_blocks[field_name]
                        label = audience_block.label or field_name
                        name = audience_block.name
                        info_field.block.child_blocks[block_type].child_blocks[field_name] = blocks.ChoiceBlock(
                            choices=audience_type_choices, required=False, help_text=AUDIENCE_HELP_TEXT)
                        info_field.block.child_blocks[block_type].child_blocks[field_name].name = name
                        info_field.block.child_blocks[block_type].child_blocks[field_name].label = label

    def clean(self):
        cleaned_data = super().clean()

        # validata msgType
        msgType = cleaned_data.get("msgType")
        if msgType and msgType != 'Alert':
            references = cleaned_data.get("references")
            if not references:
                # if the message type is not 'Alert' then references are required
                self.add_error('references', _("You must select at least one reference alert for this message type. "
                                               "Only 'Alert' Message Type can be saved without references."))
            else:
                alerts_ids = []
                for reference in references:
                    ref_alert_page = reference.value.get("ref_alert").specific
                    if ref_alert_page:
                        alerts_ids.append(ref_alert_page.identifier)

                # check if the same alert is selected more than once
                if len(alerts_ids) != len(set(alerts_ids)):
                    self.add_error('references', _("You cannot select the same alert more than once."))

        return cleaned_data


class AbstractCapAlertPage(Page):
    base_form_class = CapAlertPageForm

    exclude_fields_in_copy = ["identifier"]

    STATUS_CHOICES = (
        ("Draft", _("Draft - A preliminary template or draft, not actionable in its current form")),
        ("Actual", _("Actual - Actionable by all targeted recipients")),
        ("Test", _("Test - Technical testing only, all recipients disregard")),
        ("Exercise", _("Exercise - Actionable only by designated exercise participants; "
                       "exercise identifier SHOULD appear in note")),
        ("system", _("System - For messages that support alert network internal functions")),
    )

    MESSAGE_TYPE_CHOICES = (
        ('Alert', _("Alert - Initial information requiring attention by targeted recipients")),
        ('Update', _("Update - Updates and supersedes the earlier message(s) identified in referenced alerts")),
        ('Cancel', _("Cancel - Cancels the earlier message(s) identified in references")),
        ('Ack', _("Acknowledge - Acknowledges receipt and acceptance of the message(s) "
                  "identified in references field")),
        ('Error', _("Error -  Indicates rejection of the message(s) identified in references; "
                    "explanation SHOULD appear in note field")),
    )

    SCOPE_CHOICES = (
        ('Public', _("Public - For general dissemination to unrestricted audiences")),
        ('Restricted', _("Restricted - For dissemination only to users with a known operational "
                         "requirement as in the restriction field")),
        ('Private', _("Private - For dissemination only to specified addresses"
                      " as in the addresses field in the alert")),
    )

    identifier = models.UUIDField(default=uuid.uuid4, editable=False, verbose_name=_("Identifier"),
                                  help_text=_("Unique ID. Auto generated on creation."), unique=True)
    sender = models.CharField(max_length=255, verbose_name=_("Sender"), default=get_default_sender,
                              help_text=_("Identifies the originator of an alert. "
                                          "For example the website address of the institution"))
    sent = models.DateTimeField(default=timezone.now, verbose_name=_("Sent"),
                                help_text=_("Time and date of origination of the alert"))
    status = models.CharField(max_length=50, choices=STATUS_CHOICES, default="Actual", verbose_name=_("Status"),
                              help_text=_("The code denoting the appropriate handling of the alert"))
    msgType = models.CharField(max_length=100, choices=MESSAGE_TYPE_CHOICES, default="Alert",
                               verbose_name=_("Message Type"),
                               help_text=_("The code denoting the nature of the alert message"))
    scope = models.CharField(max_length=100, choices=SCOPE_CHOICES, default="Public", verbose_name=_("Scope"),
                             help_text=_("The code denoting the intended distribution of the alert message"))
    source = models.TextField(blank=True, null=True, verbose_name=_("Source"),
                              help_text=_("The text identifying the source of the alert message"))
    restriction = models.TextField(blank=True, null=True,
                                   help_text=_("The text describing the rule for limiting distribution of the "
                                               "restricted alert message. Used when scope value is Restricted"),
                                   verbose_name=_("Restriction"))
    code = models.CharField(max_length=100, blank=True, null=True,
                            help_text=_("The code denoting the special handling of the alert message"),
                            verbose_name=_("Code"))
    note = models.TextField(blank=True, null=True,
                            help_text=_("The text describing the purpose or significance of the alert message."
                                        "The message note is primarily intended for use with "
                                        "<status> 'Exercise' and <msgType> 'Error'"), verbose_name=_("Note"))
    info = StreamField([
        ("alert_info", AlertInfo(label=_("Alert Information")))
    ], use_json_field=True, min_num=1, max_num=1, block_counts={'alert_info': {'max_num': 1, "min_num": 1}, },
        verbose_name=_("Alert Information"), )

    addresses = StreamField([
        ("recipient", AlertAddress(label=_("Recipient")))
    ], use_json_field=True, blank=True, null=True, verbose_name=_("Addresses"),
        help_text=_("The group listing of intended recipients of the alert message, if scope is Private"))

    references = StreamField([
        ("reference", AlertReference(label=_("Reference Alert")))
    ], use_json_field=True, blank=True,
        null=True, verbose_name=_("Reference Alerts"))

    incidents = StreamField([
        ("incident", AlertIncident(label=_("Incident")))
    ], use_json_field=True, blank=True,
        null=True, verbose_name=_("Incidents"))

    class Meta:
        abstract = True

    content_panels = [
        MultiFieldPanel([
            FieldPanel('sender'),
            FieldPanel('sent'),
            FieldPanel('status'),
            FieldPanel('msgType'),
            FieldPanel("references", classname="cap-alert__panel-references"),
            FieldPanel('note', classname="cap-alert__panel-note"),
            FieldPanel('scope'),
            FieldPanel('restriction', classname="cap-alert__panel-restriction"),
        ], heading=_("Alert Identification")),
        FieldPanel("addresses", classname="cap-alert__panel-addresses"),
        FieldPanel("info"),
        FieldPanel("incidents"),
    ]

    @cached_property
    def feature_collection(self):
        fc = {"type": "FeatureCollection", "features": []}
        for info in self.info:
            if info.value.features:
                for feature in info.value.features:
                    feature.get("properties", {}).update({"info-id": info.id})
                    fc["features"].append(feature)
        return fc

    @cached_property
    def geojson(self):
        return json.dumps(self.feature_collection)

    @cached_property
    def bounds(self):
        geojson_data = self.feature_collection
        bounds = None
        for feature in geojson_data['features']:
            geometry = shape(feature['geometry'])
            if bounds is None:
                bounds = geometry.bounds
            else:
                bounds = (
                    min(bounds[0], geometry.bounds[0]),
                    min(bounds[1], geometry.bounds[1]),
                    max(bounds[2], geometry.bounds[2]),
                    max(bounds[3], geometry.bounds[3])
                )

        return list(bounds)

    @property
    def xml_link(self):
        return None

    @cached_property
    def infos(self):
        alert_infos = []
        for info in self.info:
            start_time = info.value.get("effective") or self.sent
            expired = False

            if info.value.get('expires') < timezone.localtime():
                status = "Expired"
                expired = True
            elif timezone.localtime() > start_time:
                status = "Ongoing"
            else:
                status = "Expected"

            area_desc = [area.get("areaDesc") for area in info.value.area]
            area_desc = ", ".join(area_desc)

            event = f"{info.value.get('event')} ({area_desc})"
            severity = SEVERITY_MAPPING[info.value.get("severity")]
            urgency = URGENCY_MAPPING[info.value.get("urgency")]
            certainty = CERTAINTY_MAPPING[info.value.get("certainty")]

            effective = start_time
            expires = info.value.get('expires')
            url = self.url
            event_icon = info.value.event_icon

            alert_info = {
                "info": info,
                "status": status,
                "url": self.url,
                "event": event,
                "event_icon": event_icon,
                "severity": severity,
                "utc": start_time,
                "urgency": urgency,
                "certainty": certainty,
                "sent": self.sent,
                "effective": effective,
                "expires": expires,
                "expired": expired,
                "properties": {
                    "id": self.identifier,
                    "event": event,
                    "event_type": info.value.get('event'),
                    "headline": info.value.get("headline"),
                    "severity": info.value.get("severity"),
                    "urgency": info.value.get("urgency"),
                    "certainty": info.value.get("certainty"),
                    "severity_color": severity.get("color"),
                    "sent": self.sent,
                    "onset": info.value.get("onset"),
                    "expires": expires,
                    "expired": expired,
                    "web": url,
                    "description": info.value.get("description"),
                    "instruction": info.value.get("instruction"),
                    "event_icon": event_icon,
                    "area_desc": area_desc,
                }
            }

            alert_infos.append(alert_info)

        return alert_infos

    def get_geojson_features(self, request=None):
        features = []
        for info_item in self.infos:
            info = info_item.get("info")
            if info.value.geojson:
                properties = info_item.get("properties")
                if request:
                    web = request.build_absolute_uri(properties.get("web"))
                    properties.update({
                        "web": web
                    })

                info_features = info.value.features
                for feature in info_features:
                    feature["properties"].update(**properties)
                    features.append(feature)

        return features



from django.shortcuts import redirect
from django.template.response import TemplateResponse
from django.templatetags.static import static
from django.urls import path
from django.utils.html import format_html
from django.utils.translation import gettext as _
from wagtail import hooks
from wagtail.actions.copy_page import CopyPageAction
from wagtail.admin import messages
from wagtail.admin.forms.pages import CopyForm
from wagtail.admin.utils import get_valid_next_url_from_request
from wagtail.models import Page

from capeditor.views import load_cap_alert, import_cap_alert


@hooks.register("insert_editor_js")
def insert_editor_js():
    return format_html(
        '<script src="{}"></script>', static("capeditor/js/conditional_fields.js"),
    )


@hooks.register("register_icons")
def register_icons(icons):
    return icons + [
        'capeditor/icons/cap-alert.svg',
        'capeditor/icons/cap-alert-full.svg',
    ]


@hooks.register('register_admin_urls')
def urlconf_stations():
    return [
        path('import-cap/', load_cap_alert, name='load_cap_alert'),
        path('import-cap/import/', import_cap_alert, name='import_cap_alert'),
    ]


# @hooks.register("before_copy_page")
def copy_cap_alert_page(request, page):
    if page.specific.__class__.__name__ == "CapAlertPage":

        # Parent page defaults to parent of source page
        parent_page = page.get_parent()

        # Check if the user has permission to publish subpages on the parent
        can_publish = parent_page.permissions_for_user(request.user).can_publish_subpage()

        # Create the form
        form = CopyForm(
            request.POST or None, user=request.user, page=page, can_publish=can_publish
        )

        next_url = get_valid_next_url_from_request(request)

        # Remove the publish_copies and alias fields from the form
        form.fields.pop("publish_copies")
        form.fields.pop("alias")

        # Check if user is submitting
        if request.method == "POST":
            # Prefill parent_page in case the form is invalid (as prepopulated value for the form field,
            # because ModelChoiceField seems to not fall back to the user given value)
            parent_page = Page.objects.get(id=request.POST["new_parent_page"])

            if form.is_valid():
                # Receive the parent page (this should never be empty)
                if form.cleaned_data["new_parent_page"]:
                    parent_page = form.cleaned_data["new_parent_page"]

                action = CopyPageAction(
                    page=page,
                    recursive=form.cleaned_data.get("copy_subpages"),
                    to=parent_page,
                    update_attrs={
                        "title": form.cleaned_data["new_title"],
                        "slug": form.cleaned_data["new_slug"],
                    },
                    keep_live=False,
                    copy_revisions=False,
                    user=request.user,
                )
                new_page = action.execute()

                messages.success(
                    request,
                    _("Page '%(page_title)s' copied.")
                    % {"page_title": page.specific_deferred.get_admin_display_title()},
                )

                for fn in hooks.get_hooks("after_copy_page"):
                    result = fn(request, page, new_page)
                    if hasattr(result, "status_code"):
                        return result

                # Redirect to explore of parent page
                if next_url:
                    return redirect(next_url)
                return redirect("wagtailadmin_explore", parent_page.id)

        return TemplateResponse(
            request,
            "wagtailadmin/pages/copy.html",
            {
                "page": page,
                "form": form,
                "next": next_url,
            },
        )

    return



# Generated by Django 4.1.7 on 2023-07-12 09:55

from django.db import migrations, models
import django.db.models.deletion
import wagtail.blocks
import wagtail.fields
import wagtailiconchooser.blocks


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0083_workflowcontenttype'),
    ]

    operations = [
        migrations.CreateModel(
            name='CapSetting',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sender', models.CharField(blank=True, help_text='Can be the website link or email of the sending institution', max_length=255, null=True, verbose_name='CAP Sender Identifier')),
                ('sender_name', models.CharField(blank=True, help_text='Name of the sending institution', max_length=255, null=True, verbose_name='CAP Sender Name')),
                ('contacts', wagtail.fields.StreamField([('contact', wagtail.blocks.StructBlock([('contact', wagtail.blocks.CharBlock(label='Contact Detail', max_length=255))], label='Contact'))], blank=True, help_text='Contact for follow-up and confirmation of the alert message', null=True, use_json_field=True, verbose_name='Contact Details')),
                ('hazard_types', wagtail.fields.StreamField([('hazard_type', wagtail.blocks.StructBlock([('hazard', wagtail.blocks.CharBlock(help_text='Name of Hazard', label='Hazard', max_length=255)), ('icon', wagtailiconchooser.blocks.IconChooserBlock(required=False))], label='Hazard Type'))], blank=True, help_text='Hazards monitored by the institution', null=True, use_json_field=True, verbose_name='Hazard Types')),
                ('audience_types', wagtail.fields.StreamField([('audience_type', wagtail.blocks.StructBlock([('audience', wagtail.blocks.CharBlock(help_text='Intended audience', label='Audience', max_length=255))], label='Audience Type'))], blank=True, help_text='Target audiences for published alerts', null=True, use_json_field=True, verbose_name='Audience Types')),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.site')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# wmo event types list per Cg-18
WMO_HAZARD_EVENTS_TYPE_CHOICES = (
    ("Avalanche", "Avalanche"),
    ("Cold wave", "Cold wave"),
    ("Drought/Dry spell", "Drought/Dry spell"),
    ("Dust storm/Sandstorm", "Dust storm/Sandstorm"),
    ("Extra-tropical cyclone", "Extra-tropical cyclone"),
    ("Flood", "Flood"),
    ("Fog", "Fog"),
    ("Haze/Smoke", "Haze/Smoke"),
    ("Frost", "Frost"),
    ("Hail", "Hail"),
    ("Heat wave", "Heat wave"),
    ("High UV radiation", "High UV radiation"),
    ("Icing", "Icing"),
    ("Freezing rain", "Freezing rain"),
    ("Landslide/Mudslide & Debris flow", "Landslide/Mudslide & Debris flow"),
    ("Lighting", "Lighting"),
    ("Pollen pollution/Polluted air", "Pollen pollution/Polluted air"),
    ("Rain/Wet Spell", "Rain/Wet Spell"),
    ("Snow", "Snow"),
    ("Snowstorm", "Snowstorm"),
    ("Space weather event", "Space weather event"),
    ("High Seas/Rogue waves etc.", "High Seas/Rogue waves etc."),
    ("Storm surge/Coastal flood", "Storm surge/Coastal flood"),
    ("Thunderstorms/Squall lines", "Thunderstorms/Squall lines"),
    ("Tornado", "Tornado"),
    ("Tropical cyclone", "Tropical cyclone"),
    ("Tsunami", "Tsunami"),
    ("Volcanic ash", "Volcanic ash"),
    ("Wild land fire/Forest fire", "Wild land fire/Forest fire"),
    ("Wind", "Wind"),
)

CAP_MESSAGE_ORDER_SEQUENCE = {
    "alert": [
        "identifier",
        "sender",
        "sent",
        "status",
        "msgType",
        "source",
        "scope",
        "restriction",
        "addresses",
        "code",
        "note",
        "references",
        "incidents",
        "info"
    ],
    "info": [
        "language",
        "category",
        "event",
        "responseType",
        "urgency",
        "severity",
        "certainty",
        "audience",
        "eventCode",
        "effective",
        "onset",
        "expires",
        "senderName",
        "headline",
        "description",
        "instruction",
        "web",
        "contact",
        "parameter",
        "resource",
        "area"
    ],
    "resource": [
        "resourceDesc",
        "mimeType",
        "size",
        "uri",
        "derefUri",
        "digest",
    ],
    "area": [
        "areaDesc",
        "polygon",
        "polygons",  # added to support multiple polygons. Not in CAP spec
        "circle",
        "geocode",
        "altitude",
        "ceiling"
    ]
}

SEVERITY_MAPPING = {
    "Extreme": {
        "label": "Red severity",
        "color": "#d72f2a",
        "background_color": "#fcf2f2",
        "border_color": "#721515",
        "icon_color": "#fff",
        "severity": "Extreme",
        "id": 4
    },
    "Severe": {
        "label": "Orange severity",
        "color": "#fe9900",
        "background_color": "#fff9f2",
        "border_color": "#9a6100",
        "severity": "Severe",
        "id": 3
    },
    "Moderate": {
        "label": "Yellow severity",
        "color": "#ffff00",
        "background_color": "#fffdf1",
        "border_color": "#938616",
        "severity": "Moderate",
        "id": 2
    },
    "Minor": {
        "label": "Minor severity",
        "color": "#03ffff",
        "background_color": "#fffdf1",
        "border_color": "#938616",
        "severity": "Minor",
        "id": 1
    },
    "Unknown": {
        "label": "Unknown severity",
        "color": "#3366ff",
        "background_color": "#fffdf1",
        "border_color": "#122663",
        "icon_color": "#fff",
        "severity": "Unknown",
        "id": 0
    }
}

URGENCY_MAPPING = {
    "Immediate": {
        "label": "Immediate",
        "certainty": "Immediate",
        "id": 4
    },
    "Expected": {
        "label": "Expected",
        "certainty": "Expected",
        "id": 3
    },
    "Future": {
        "label": "Future",
        "certainty": "Future",
        "id": 2
    },
    "Past": {
        "label": "Past",
        "certainty": "Past",
        "id": 1
    },
    "Unknown": {
        "label": "Unknown",
        "certainty": "Unknown",
        "id": 0
    },
}

CERTAINTY_MAPPING = {
    "Observed": {
        "label": "Observed",
        "certainty": "Observed",
        "id": 4
    },
    "Likely": {
        "label": "Likely",
        "certainty": "Likely",
        "id": 3
    },
    "Possible": {
        "label": "Possible",
        "certainty": "Possible",
        "id": 2
    },
    "Unlikely": {
        "label": "Unlikely",
        "certainty": "Unlikely",
        "id": 1
    },
    "Unknown": {
        "label": "Unknown",
        "certainty": "Unknown",
        "id": 0
    },
}



import json

from adminboundarymanager.models import AdminBoundarySettings
from django.contrib.gis.forms import BaseGeometryWidget
from django.contrib.gis.geometry import json_regex
from django.forms import Textarea, Widget, TextInput
from django.urls import reverse
from wagtail.models import Site
from wagtail.telepath import register
from wagtail.utils.widgets import WidgetWithScript
from wagtail.widget_adapters import WidgetAdapter

from capeditor.constants import WMO_HAZARD_EVENTS_TYPE_CHOICES


class BaseMapWidget(Widget):
    def get_context(self, *args, **kwargs):
        context = super().get_context(*args, **kwargs)

        try:
            site = Site.objects.get(is_default_site=True)
            abm_settings = AdminBoundarySettings.for_site(site)
            boundary_tiles_url = abm_settings.boundary_tiles_url
            boundary_tiles_url = site.root_url + boundary_tiles_url

            boundary_detail_url = reverse("admin_boundary_detail", args=[0]).replace("/0", "")
            boundary_detail_url = site.root_url + boundary_detail_url

            context.update({
                "boundary_tiles_url": boundary_tiles_url,
                "boundary_detail_url": boundary_detail_url,
                "country_bounds": abm_settings.combined_countries_bounds
            })

        except Exception:
            pass

        return context


class BasePolygonWidget(BaseGeometryWidget, BaseMapWidget):
    def serialize(self, value):
        return value.json if value else ""


class BoundaryPolygonWidget(WidgetWithScript, BasePolygonWidget):
    template_name = "capeditor/widgets/boundary_polygon_widget.html"
    map_srid = 4326

    def __init__(self, attrs=None):
        default_attrs = {
            "class": "capeditor-widget__boundary-input",
        }
        attrs = attrs or {}
        attrs = {**default_attrs, **attrs}

        super().__init__(attrs=attrs)

    class Media:
        css = {
            "all": [
                "capeditor/css/cap_detail_page.css",
                "capeditor/css/widget/boundary-widget.css",
            ]
        }
        js = [
            "capeditor/js/maplibre-gl.js",
            "capeditor/js/turf.min.js",
            "capeditor/js/widget/boundary-polygon-widget.js",
        ]


class BoundaryPolygonWidgetAdapter(WidgetAdapter):
    js_constructor = "capeditor.widgets.BoundaryPolygonInput"

    class Media:
        js = [
            "capeditor/js/widget/boundary-polygon-widget-telepath.js",
        ]


register(BoundaryPolygonWidgetAdapter(), BoundaryPolygonWidget)


class PolygonWidget(WidgetWithScript, BasePolygonWidget):
    template_name = "capeditor/widgets/polygon_widget.html"
    map_srid = 4326

    def __init__(self, attrs=None):
        default_attrs = {
            "class": "capeditor-widget__polygon-input",
        }
        attrs = attrs or {}
        attrs = {**default_attrs, **attrs}

        super().__init__(attrs=attrs)

    class Media:
        css = {
            "all": [
                "capeditor/css/maplibre-gl.css",
                "capeditor/css/mapbox-gl-draw.css",
                "capeditor/css/widget/polygon-widget.css",
            ]
        }
        js = [
            "capeditor/js/maplibre-gl.js",
            "capeditor/js/mapbox-gl-draw.js",
            "capeditor/js/turf.min.js",
            "capeditor/js/widget/polygon-widget.js",
        ]


class PolygonWidgetAdapter(WidgetAdapter):
    js_constructor = "capeditor.widgets.PolygonInput"

    class Media:
        js = [
            "capeditor/js/widget/polygon-widget-telepath.js",
        ]


register(PolygonWidgetAdapter(), PolygonWidget)


class CircleWidget(WidgetWithScript, BaseMapWidget, Textarea):
    template_name = "capeditor/widgets/circle_widget.html"

    def __init__(self, attrs=None):
        default_attrs = {
            "class": "capeditor-widget__circle-input",
        }

        if attrs:
            default_attrs.update(attrs)

        super().__init__(default_attrs)

    class Media:
        css = {
            "all": [
                "capeditor/css/maplibre-gl.css",
                "capeditor/css/widget/circle-widget.css",
            ]
        }
        js = [
            "capeditor/js/maplibre-gl.js",
            "capeditor/js/turf.min.js",
            "capeditor/js/widget/circle-widget.js",
        ]


class CircleWidgetAdapter(WidgetAdapter):
    js_constructor = "capeditor.widgets.CircleInput"

    class Media:
        js = [
            "capeditor/js/widget/circle-widget-telepath.js",
        ]


register(CircleWidgetAdapter(), CircleWidget)


class HazardEventTypeWidget(WidgetWithScript, TextInput):
    template_name = "capeditor/widgets/hazard_event_type_widget.html"

    def __init__(self, attrs=None, **kwargs):
        default_attrs = {}

        if attrs:
            default_attrs.update(attrs)

        super().__init__(default_attrs)

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)

        options = []

        for choice in WMO_HAZARD_EVENTS_TYPE_CHOICES:
            options.append({
                "value": choice[0],
                "label": choice[1],
            })

        context["widget"].update({
            "event_type_list": options
        })

        return context

    def render_js_init(self, id_, name, value):
        return "new HazardEventTypeWidget({0},{1});".format(json.dumps(id_), json.dumps(value))

    class Media:
        js = [
            "capeditor/js/widget/hazard-event-type-widget.js",
        ]


class PolygonDrawWidget(BaseGeometryWidget, BaseMapWidget):
    template_name = "capeditor/widgets/polygon_draw_widget.html"
    map_srid = 4326

    def __init__(self, attrs=None):
        default_attrs = {
            "class": "capeditor-widget__polygon-draw-input",
        }
        attrs = attrs or {}
        attrs = {**default_attrs, **attrs}

        super().__init__(attrs=attrs)

    class Media:
        css = {
            "all": [
                "capeditor/css/maplibre-gl.css",
                "capeditor/css/mapbox-gl-draw.css",
                "capeditor/css/widget/polygon-draw-widget.css",
            ]
        }
        js = [
            "capeditor/js/maplibre-gl.js",
            "capeditor/js/mapbox-gl-draw.js",
            "capeditor/js/turf.min.js",
            "capeditor/js/widget/polygon-draw-widget.js",
        ]

    def serialize(self, value):
        return value.json if value else ""

    def deserialize(self, value):
        geom = super().deserialize(value)
        # GeoJSON assumes WGS84 (4326). Use the map's SRID instead.
        if geom and json_regex.match(value) and self.map_srid != 4326:
            geom.srid = self.map_srid
        return geom



from django.contrib.gis.forms import GeometryField as BaseGeometryField
from django.contrib.gis.geos import GEOSException
from django.core.exceptions import ValidationError

from .widgets import PolygonWidget, BoundaryPolygonWidget


class MultiPolygonGeometryField(BaseGeometryField):
    geom_type = 'MULTIPOLYGON'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.widget.geom_type = self.geom_type


class PolygonGeometryField(BaseGeometryField):
    geom_type = 'POLYGON'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.widget.geom_type = self.geom_type


class BoundaryMultiPolygonField(MultiPolygonGeometryField):
    widget = BoundaryPolygonWidget


class MultiPolygonField(MultiPolygonGeometryField):
    widget = PolygonWidget


class PolygonOrMultiPolygonField(BaseGeometryField):
    widget = PolygonWidget

    def clean(self, value):
        """
        Validate that the input value can be converted to a Geometry object
        and return it. Raise a ValidationError if the value cannot be
        instantiated as a Geometry.
        """
        geom = super(BaseGeometryField, self).clean(value)
        if geom is None:
            return geom

        # Ensuring that the geometry is of the correct type (indicated
        # using the OGC string label).
        val_geom_type = str(geom.geom_type).upper()

        ALLOWED_GEOM_TYPES = ["POLYGON", "MULTIPOLYGON"]

        if val_geom_type not in ALLOWED_GEOM_TYPES:
            raise ValidationError(
                self.error_messages["invalid_geom_type"], code="invalid_geom_type"
            )

        # Transforming the geometry if the SRID was set.
        if self.srid and self.srid != -1 and self.srid != geom.srid:
            try:
                geom.transform(self.srid)
            except GEOSException:
                raise ValidationError(
                    self.error_messages["transform_error"], code="transform_error"
                )

        return geom


class PolygonField(PolygonGeometryField):
    widget = PolygonWidget



import pytz
from dateutil.parser import isoparse
from rest_framework import serializers

from capeditor.constants import CAP_MESSAGE_ORDER_SEQUENCE
from capeditor.utils import order_dict_by_keys


def parse_tz(date_str):
    dt = isoparse(date_str)
    # check if we have timezone info
    if dt.tzinfo:
        # check if timezone is equal to UTC, and replace '+' with '-' to match CAP Protocol time formats
        if dt.utcoffset().total_seconds() == 0:
            date_str = dt.isoformat().replace("+", "-")
        else:
            date_str = dt.isoformat()
    # NO timezone. Assume is UTC
    else:
        date_str = dt.replace(tzinfo=pytz.UTC).isoformat().replace("+", "-")

    return date_str


class AlertSerializer(serializers.ModelSerializer):
    addresses = serializers.SerializerMethodField()
    references = serializers.SerializerMethodField()
    info = serializers.SerializerMethodField()
    incidents = serializers.SerializerMethodField()

    class Meta:
        fields = [
            "identifier",
            "sender",
            "sent",
            "status",
            "msgType",
            "source",
            "scope",
            "restriction",
            "addresses",
            "code",
            "note",
            "references",
            "incidents",
            "info",
        ]

    def get_info(self, obj):
        request = self.context.get("request")
        info_values = []

        for info in obj.info:
            info_obj = info.block.get_api_representation(info.value)
            if info.value.resource:
                resources = []
                for resource in info.value.resource:
                    if resource.get("type") == "doc":
                        resource["uri"] = request.build_absolute_uri(resource.get("uri"))
                    resource.pop("type")
                    # order resource according to CAP_MESSAGE_ORDER_SEQUENCE
                    resource_obj = order_dict_by_keys(resource, CAP_MESSAGE_ORDER_SEQUENCE.get("resource"))
                    resources.append(resource_obj)
                info_obj["resource"] = resources

            # assign full url
            info_obj["web"] = obj.get_full_url(request)
            for field in list(info_obj):
                if not info_obj[field]:
                    info_obj.pop(field)
            area_values = []

            # format responseTypes
            if info_obj.get("responseType"):
                response_types = []
                for response_type in info_obj.get("responseType"):
                    response_types.append(response_type.get("response_type"))
                info_obj["responseType"] = response_types

            # format area
            if info.value.area:
                areas = []
                for area in info.value.area:
                    area_obj = order_dict_by_keys(area, CAP_MESSAGE_ORDER_SEQUENCE.get("area"))
                    areas.append(area_obj)
                info_obj["area"] = areas

            # order according to CAP_MESSAGE_ORDER_SEQUENCE
            info_obj = order_dict_by_keys(info_obj, CAP_MESSAGE_ORDER_SEQUENCE.get("info"))

            info_values.append(info_obj)

        return info_values

    @staticmethod
    def get_addresses(obj):
        address_values = []
        for address in obj.addresses:
            address_obj = address.block.get_api_representation(address.value)
            address_values.append(address_obj.get("address"))

        if address_values:
            return " ".join(address_values)

        return None

    @staticmethod
    def get_references(obj):
        reference_values = []
        for reference in obj.references:
            if reference.value.ref_alert_identifier:
                reference_values.append(reference.value.ref_alert_identifier)
        if reference_values:
            return " ".join(reference_values)
        return None

    @staticmethod
    def get_incidents(obj):
        incident_values = []
        for incident in obj.incidents:
            incident_obj = incident.block.get_api_representation(incident.value)
            incident_val = incident_obj.get('incident')
            incident_values.append(f'"{incident_val}"')

        if incident_values:
            return " ".join(incident_values)

        return None

    def to_representation(self, instance):
        representation = super().to_representation(instance)

        representation['sent'] = parse_tz(representation['sent'])

        # check for scope condition
        if representation[f'scope'] == 'Restricted':
            representation[f'addresses'] = None
        elif representation[f'scope'] == 'Private':
            representation[f'restriction'] = None
        elif representation[f'scope'] == 'Public':
            representation[f'restriction'] = None
            representation[f'addresses'] = None

        # check message type
        if representation['msgType'] == 'Update' or representation['msgType'] == 'Cancel' or \
                representation['msgType'] == 'Ack':
            representation[f'note'] = None
        elif representation[f'msgType'] == 'Alert':
            representation[f'note'] = None
            representation[f'references'] = None

        return {k: v for k, v in representation.items() if v or v == 0 or v is False}
